@@ -191,6 +191,12 @@ ColorSpace GlRenderer::colorSpace()
 }
 
 
+const Surface* GlRenderer::mainSurface()
+{
+    return &surface;
+}
+
+
 bool GlRenderer::blend(TVG_UNUSED BlendMethod method)
 {
     //TODO:
@@ -405,7 +411,7 @@ RenderData GlRenderer::prepare(const RenderShape& rshape, RenderData data, const
 
 RenderRegion GlRenderer::viewport()
 {
-    return {0, 0, static_cast<int32_t>(surface.w), static_cast<int32_t>(surface.h)};
+    return mViewport;
 }
 
 
@@ -449,7 +455,7 @@ GlRenderer* GlRenderer::gen()
     return new GlRenderer();
 }
 
-GlRenderer::GlRenderer() :mViewport() ,mGpuBuffer(new GlStageBuffer), mPrograms(), mComposePool()
+GlRenderer::GlRenderer() :mGpuBuffer(new GlStageBuffer), mPrograms(), mComposePool()
 {
 }
 
